import 'dart:async';

import 'package:fluid/fluid.dart';

class DebounceFlow<T> extends Flow<T> {
  final Flow<T> _source;
  final Duration _window;

  Timer? _debounceTimer;

  DebounceFlow(
    this._source,
    this._window,
  ) : super(_source.value) {
    _source.addListener(_onFlowUpdate);
  }

  void _onFlowUpdate() {
    _debounceTimer?.cancel();
    final updatedValue = _source.value;
    _debounceTimer = Timer(_window, () {
      emit(updatedValue);
    });
  }

  @override
  void dispose() {
    _source.removeListener(_onFlowUpdate);
    super.dispose();
  }
}
