import 'dart:async';

import 'package:flutter/foundation.dart';

class Flow<T> with ChangeNotifier {
  T _value;
  T get value => _value;

  Flow(this._value);

  void emit(T newValue) {
    _value = newValue;
    notifyListeners();
  }
}

extension FlowExtensions<T> on Flow<T> {
  Flow<T2> map<T2>(FlowMapper<T, T2> mapper) {
    return MappingFlow(this, mapper);
  }

  Flow<T> debounce(Duration window) {
    return DebounceFlow(this, window);
  }

  Flow<T> distinct() {
    return DistinctFlow(this);
  }
}

extension StreamExtension<T> on Stream<T> {
  Flow<T> asFlow(T initialValue) {
    return StreamFlow(this, initialValue);
  }
}
