import 'dart:async';

import 'package:flutter/foundation.dart';

typedef StreamObserver<T> = void Function(T value);

abstract class Fluid {
  final List<StreamSubscription> _streamSubscriptions = [];

  @protected
  ValueListenable<T> valueNotifier<T>(Stream<T> stream, T initialValue) {
    final valueNotifier = ValueNotifier(initialValue);
    final subscription = stream.listen((event) {
      if (valueNotifier.value != event) {
        valueNotifier.value = event;
      }
    });
    _streamSubscriptions.add(subscription);
    return valueNotifier;
  }

  @protected
  void observe<T>(
    Stream<T> stream,
    StreamObserver<T> observer,
  ) {
    final subscription = stream.listen((data) {
      observer(data);
    });
    _streamSubscriptions.add(subscription);
  }

  void dispose() {
    for (final subscription in _streamSubscriptions) {
      subscription.cancel();
    }
    _streamSubscriptions.clear();
  }
}
