import 'package:fluid/fluid.dart';
import 'package:flutter/widgets.dart';

typedef FluidChildBuilder<T extends Fluid> = Widget Function(
  BuildContext context,
  T fluid,
);

class FluidBuilder<T extends Fluid> extends StatefulWidget {
  final FluidFactory<T> fluidFactory;
  final FluidChildBuilder<T> builder;
  const FluidBuilder({
    required this.builder,
    required this.fluidFactory,
    Key? key,
  }) : super(key: key);

  @override
  State<FluidBuilder<T>> createState() => _FluidBuilderState<T>();
}

class _FluidBuilderState<T extends Fluid> extends State<FluidBuilder<T>> {
  late T fluid;
  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    final fluidResolver = FluidResolver.of(context);
    fluid = fluidResolver.getFluid(widget.fluidFactory);
  }

  @override
  void dispose() {
    fluid.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final child = widget.builder(context, fluid);
    return child;
  }
}
